/* REPOS -- by Mark Cederholm

 * Corrects the geopositioning information of a CONCATenated image
   based on the .DOC files of the input images

 * Version 1.0, 4/1/98

 * Usage:  REPOS [cat_doc] [i_doc1] [i_doc2] ... [i_docn]

 * Arguments:  cat_doc = output doc file (.DOC extension assumed)
               i_doc1 ... i_docn = input doc files (.DOC extension assumed)

/*********************************************************************/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

int main(int argc, char *argv[])
{

   /**** initialize environment **************************************/

   FILE *infile, *outfile;
   char doc[81];         /* .DOC file name                           */
   char bak[81];         /* .BAK file name                           */
   char input[81];       /* input line                               */
   char temp[81];        /* working string                           */
   char *tok;            /* current token                            */
   double x, y;          /* current left and top values              */
   double cxmin, cymax;  /* leftmost and topmost values in cat_doc   */
   double xmin, ymax;    /* true leftmost and topmost values         */
   double val;           /* output value                             */
   int i, tmp;           /* temp variables                           */

   /* check # of arguments */

   if (argc < 3)
   {
      printf("Usage:  REPOS [cat_doc] [i_doc1] [i_doc2] ... [i_docn]\n");
      return 0;
   }

   /* read xmin and ymax values */

   i = 1;
   while (i < argc)
   {

      /* open file */

      strcpy(doc, argv[i]);
      strcat(doc, ".doc");
      if ((infile = fopen(doc,"r")) == NULL)
      {
         printf("Error opening input file %s.\n", doc);
         return 0;
      }

      /* read Xmin and Ymax values */

      while (fgets(input, 80, infile) != NULL)
      {
         strcpy(temp,"min. X");
         tmp = strlen(temp);
         if (strncmp(input,temp,tmp) == 0)
         {
            strcpy(temp,input);
            tok = strtok(temp, " \n");
            tok = strtok(NULL, " \n");
            tok = strtok(NULL, " \n");
            tok = strtok(NULL, " \n");
            x = atof(tok);
            continue;
         }

         strcpy(temp,"max. Y");
         tmp = strlen(temp);
         if (strncmp(input,temp,tmp) == 0)
         {
            strcpy(temp,input);
            tok = strtok(temp, " \n");
            tok = strtok(NULL, " \n");
            tok = strtok(NULL, " \n");
            tok = strtok(NULL, " \n");
            y = atof(tok);
            continue;
         }
      }

      fclose(infile);

      if (i == 1)
      {
         cxmin = xmin = x;
         cymax = ymax = y;
      }
      else
      {
         if (x < xmin)
            xmin = x;
         if (y > ymax)
            ymax = y;
      }
      i++;
   }

   /* set up input and output files */

   strcpy(doc, argv[1]);
   strcpy(bak, argv[1]);
   strcat(doc, ".doc");
   strcat(bak, ".bak");

   if (rename(doc, bak) != 0)
   {
      printf("Error:  could not rename %s to %s.\n", doc, bak);
      return 0;
   }

   if ((infile = fopen(bak,"r")) == NULL)
   {
      printf("Error opening input file %s.\n", doc);
      return 0;
   }

   if ((outfile = fopen(doc,"w")) == NULL)
   {
      printf("Error opening output file %s.\n", doc);
      return 0;
   }

   /* process DOC file */

   while (fgets(input, 80, infile) != NULL)
   {
      strcpy(temp,"min. X");
      tmp = strlen(temp);
      if (strncmp(input,temp,tmp) == 0)
      {
         fprintf(outfile,"min. X      : %f\n", xmin);
         continue;
      }

      strcpy(temp,"max. X");
      tmp = strlen(temp);
      if (strncmp(input,temp,tmp) == 0)
      {
         strcpy(temp,input);
         tok = strtok(temp, " \n");
         tok = strtok(NULL, " \n");
         tok = strtok(NULL, " \n");
         tok = strtok(NULL, " \n");
         x = atof(tok);
         val = x + ( xmin - cxmin );
         fprintf(outfile,"max. X      : %f\n", val);
         continue;
      }

      strcpy(temp,"min. Y");
      tmp = strlen(temp);
      if (strncmp(input,temp,tmp) == 0)
      {
         strcpy(temp,input);
         tok = strtok(temp, " \n");
         tok = strtok(NULL, " \n");
         tok = strtok(NULL, " \n");
         tok = strtok(NULL, " \n");
         y = atof(tok);
         val = y + ( ymax - cymax );
         fprintf(outfile,"min. Y      : %f\n", val);
         continue;
      }

      strcpy(temp,"max. Y");
      tmp = strlen(temp);
      if (strncmp(input,temp,tmp) == 0)
      {
         fprintf(outfile,"max. Y      : %f\n", ymax);
         continue;
      }

      fprintf(outfile,"%s", input);
   }

   /* delete .BAK file */

   if (remove(bak) != 0)
   {
      printf("Error:  could not delete %s.\n", bak);
      return 0;
   }

   return 0;

}

